proc format ;
  value treatf   1 = "acar"
                 2 = "benf"
                 3 = "metf"
                 4 = "migl"
                 5 = "piog"
                 6 = "rosi"
                 7 = "sita"
                 8 = "sulf" 
                 9 = "vild" 
                10 = "plac" ;
  value studlabf 1 = "DeFronzo1995"
                 2 = "Lewin2007"
                 3 = "Willms1999"
                 4 = "Davidson2007"
                 5 = "Wolffenbuttel1999"
                 6 = "Kipnes2001"
                 7 = "Kerenyi2004"
                 8 = "Hanefeld2004"
                 9 = "Derosa2004"
                10 = "Baksi2004"
                11 = "Rosenstock2008"
                12 = "Zhu2003"
                13 = "Yang2003"
                14 = "Vongthavaravat2002"
                15 = "Oyama2008"
                16 = "Costa1997"
                17 = "Hermansen2007"
                18 = "Garber2008"
                19 = "Alex1998"
                20 = "Johnston1994"
                21 = "Johnston1998a"
                22 = "Kim2007"
                23 = "Johnston1998b"
                24 = "Gonzalez-Ortiz2004"
                25 = "Stucci1996"
                26 = "Moulin2006" ;
run ;

data Senn2013 ;
  input TE seTE treat1 treat2 studlab ;
  format treat1 treatf. treat2 treatf. studlab studlabf. ;
  cards ;
    -1.90   0.1414  3  10   1
    -0.82   0.0992  3  10   2
    -0.20   0.3579  3   1   3
    -1.34   0.1435  6  10   4
    -1.10   0.1141  6  10   5
    -1.30   0.1268  5  10   6
    -0.77   0.1078  6  10   7
     0.16   0.0849  5   3   8
     0.10   0.1831  5   6   9
    -1.30   0.1014  6  10  10
    -1.09   0.2263  6  10  11
    -1.50   0.1624  6  10  12
    -0.14   0.2239  6   3  13
    -1.20   0.1436  6   8  14
    -0.40   0.1549  1   8  15
    -0.80   0.1432  1  10  16
    -0.57   0.1291  7  10  17
    -0.70   0.1273  9  10  18
    -0.37   0.1184  3   8  19
    -0.74   0.1839  4  10  20
    -1.41   0.2235  4  10  21
     0.00   0.2339  6   3  22
    -0.68   0.2828  4  10  23
    -0.40   0.4356  3  10  24
    -0.23   0.3467  2  10  25
    -1.01   0.1366  2  10  26
    -1.20   0.3758  3  10   3
    -1.00   0.4669  1  10   3
  ;
run ;

proc sort data=Senn2013 ;
  by studlab ;
run ;

proc transpose data=Senn2013 out=TE(rename=(col1=TE1 col2=TE2 col3=TE3) drop=_name_) ;
  by  studlab ;
  var TE ;
run ;

proc transpose data=Senn2013 out=seTE(rename=(col1=seTE1 col2=seTE2 col3=seTE3) drop=_name_) ;
  by  studlab ;
  var seTE ;
run ;

proc transpose data=Senn2013 out=treat1(rename=(col1=treat1_1 col2=treat1_2 col3=treat1_3) drop=_name_) ;
  by  studlab ;
  var treat1 ;
run ;

proc transpose data=Senn2013 out=treat2(rename=(col1=treat2_1 col2=treat2_2 col3=treat2_3) drop=_name_) ;
  by  studlab ;
  var treat2 ;
run ;

data Senn ;
  merge TE seTE treat1 treat2 ;
  by studlab ;

  v1=seTE1**2 ;
  v2=seTE2**2 ;
  v3=seTE3**2 ;
run ;

*--- fixed effect model ;
ods graphics on ;
proc mcmc data=Senn nbi=5000 nmc=500000 thin=10  seed=777 missing=ac diagnostics=all plots=all monitor=(theta) ;
  array te[2] te1 te2 ;
  array theta[9] ;
  array s[2,2] ;
  array mu[2] mu1 mu2 ;
  parms theta: 0 ;
  prior theta: ~ normal(0,var=10000) ;

  if studlab=3 then do ;
    do i=1 to 9 ;
      if treat1_1=i then mu1_1=theta[i] ;
      if treat2_1=i then mu2_1=theta[i] ;
      if treat1_2=i then mu1_2=theta[i] ;
      if treat2_2=i then mu2_2=theta[i] ;
    end ;
    if treat1_1=10 then mu1_1=0 ;
    if treat2_1=10 then mu2_1=0 ;
    if treat1_2=10 then mu1_2=0 ;
    if treat2_2=10 then mu2_2=0 ;
    
    mu[1]=mu1_1-mu2_1 ; mu[2]=mu1_2-mu2_2 ;
    s[1,1]=v1 ; s[2,2]=v2 ; s[1,2]=(v1+v2-v3)/2 ; s[2,1]=s[1,2] ;

    ll=lpdfmvn(te,mu,s) ;
  end ;
  else do ;
    do i=1 to 9 ;
      if treat1_1=i then mu1_1=theta[i] ;
      if treat2_1=i then mu2_1=theta[i] ;
    end ;
    if treat1_1=10 then mu1_1=0 ;
    if treat2_1=10 then mu2_1=0 ;

    mu[1]=mu1_1-mu2_1 ;

    ll=lpdfnorm(te[1],mu[1],sqrt(v1)) ;
  end ;

  model general(ll) ;
run ;
ods graphics off ;

*--- random effect model ;
ods graphics on ;
proc mcmc data=Senn nbi=10000 nmc=5000000 thin=50  seed=777 missing=ac diagnostics=all plots=all stats(percent=(2.5 97.5))=all monitor=(theta var_h sd) ;
  array te[2] te1 te2 ;
  array theta[9] ;
  array s[2,2] ;
  array g[2,2] ;
  array mu[2] mu1 mu2 ;
  array delta[2] delta1 delta2 ;
  parms theta: 0 ;
  parms var_h 1;

  prior theta: ~ normal(0,var=10000) ;
  prior var_h  ~ igamma(1,scale=0.000001) ;

  if studlab=3 then do ;
    do i=1 to 9 ;
      if treat1_1=i then mu1_1=theta[i] ;
      if treat2_1=i then mu2_1=theta[i] ;
      if treat1_2=i then mu1_2=theta[i] ;
      if treat2_2=i then mu2_2=theta[i] ;
    end ;
    if treat1_1=10 then mu1_1=0 ;
    if treat2_1=10 then mu2_1=0 ;
    if treat1_2=10 then mu1_2=0 ;
    if treat2_2=10 then mu2_2=0 ;

    mu[1]=mu1_1-mu2_1 ; mu[2]=mu1_2-mu2_2 ;
    s[1,1]=v1    ; s[2,2]=v2     ; s[1,2]=(v1+v2-v3)/2 ; s[2,1]=s[1,2] ;
    g[1,1]=var_h ; g[2,2]=g[1,1] ; g[1,2]=var_h/2      ; g[2,1]=g[1,2] ;

    random delta ~ mvn(mu,g) subject=_obs_ ;
    ll=lpdfmvn(te,delta,s) ;
  end ;
  else do ;
    do i=1 to 9 ;
      if treat1_1=i then mu1_1=theta[i] ;
      if treat2_1=i then mu2_1=theta[i] ;
    end ;
    if treat1_1=10 then mu1_1=0 ;
    if treat2_1=10 then mu2_1=0 ;

    mu[1]=mu1_1-mu2_1 ;
    vt=sqrt(v1) ;

    random delta3 ~ normal(mu[1],v=var_h) subject=_obs_ ;
    ll=lpdfnorm(te[1],delta3,vt) ;
  end ;

  model general(ll) ;

  sd=sqrt(var_h) ;
run ;
ods graphics off ;
