/* ******************************************************** */
/*                                                          */
/* Title:   Network Meta-Analysis using Frequentist Methods */
/* Version: 1.0                                             */
/* Date:    2016.05.19                                      */
/* License: GPL (>= 2)                                      */
/* Reference:                                               */
/*   Rucker G, Schwarzer G, Krahn U, Konig J (2015)         */
/*    netmeta: Network meta-analysis with R.                */
/*    R package version 0.8-0.                              */
/*                                                          */
/* ******************************************************** */

%macro MYNETMETA(dataset   = ,
                 sm        = MD,
                 level     = 0.95,
                 reference = ,
                 seq       = ,
                 small     = good,
                 path      = C:/temp) ;
 options noxwait xsync ;

 %let Rexepath='C:\Program Files\R\R-3.2.5\bin\i386\R.exe' ; * R.exe ̃pXw!! ;
 %let datapath=&path./data.csv   ;
 %let pdfpath =&path./output.pdf ;
 %let textpath=&path./output.txt ;
 %let prgpath =&path./program.R  ;

 options validvarname=upcase ;
 proc export data=&dataset. outfile="&datapath." dbms=csv replace ; run ;
 options validvarname=v7 ;

 filename xxx "&prgpath." ;
 data _null_ ;
  file xxx ;
  put "library(netmeta); setwd(""&path.""); options(width=1000)" ;
  put "d <- read.csv('data.csv', head=T)" ;
  put "x <- netmeta(d$TE, d$SETE, d$TREAT1, d$TREAT2, d$STUDLAB, sm=""&sm.""," ;
  put " level=&level., level.comb=&level., comb.fixed=TRUE, comb.random=TRUE," ;
  put " reference.group=""&reference."", seq=c(&seq.), warn=FALSE)" ;
  put "d1 <- decomp.design(x)" ;
  put "d2 <- netmeta:::decomp.tau(x, tau=x$tau)" ;
  put "r2 <- netrank(x, small=""&small."")" ;
  put "pdf('output.pdf', paper='a4')" ;
  put "forest(x, pooled='fixed',  reference.group=""&reference."")" ;
  put "forest(x, pooled='random', reference.group=""&reference."")" ;
  put "netgraph(x, thickness='se.fixed');  title('Fixed Effect Model')" ;
  put "netgraph(x, thickness='se.random'); title('Random Effect Model')" ;
  put "netheat(x, random=F); mtext('Fixed Effect Model', 1, 1, cex=1.8)" ;
  put "netheat(x,random=T,tau=x$tau);mtext('Random Effect Model',1,1,cex=1.8)" ;
  put "dev.off(); x$TE.random <- x$TE.fixed; x$pval.random <- x$pval.fixed" ;
  put "r1 <- netrank(x, small=""&small."")" ;
  put "write.csv(x$TE.fixed,      'TEfixed.csv', quote=F)" ;
  put "write.csv(x$lower.fixed,   'TEfixedLCL.csv', quote=F)" ;
  put "write.csv(x$upper.fixed,   'TEfixedUCL.csv', quote=F)" ;
  put "write.csv(x$TE.random,     'TErandom.csv', quote=F)" ;
  put "write.csv(x$lower.random,  'TErandomLCL.csv', quote=F)" ;
  put "write.csv(x$upper.random,  'TErandomUCL.csv', quote=F)" ;
  put "write.csv(d1$Q.decomp,     'Q1statistics.csv', quote=F)" ;
  put "write.csv(d1$Q.het.design, 'Q1withindesigns.csv', quote=F)" ;
  put "write.csv(d1$Q.inc.detach, 'Q1betweendesigns.csv', quote=F)" ;
  put "write.csv(d2$Q.decomp,     'Q2statistics.csv', quote=F)" ;
  put "write.csv(d2$Q.het.design, 'Q2withindesigns.csv', quote=F)" ;
  put "write.csv(d2$Q.inc.detach, 'Q2betweendesigns.csv', quote=F)" ;
  put "write.csv(r1$Pscore,       'rankfixed.csv', quote=F)" ;
  put "write.csv(r2$Pscore,       'rankrandom.csv', quote=F)" ;
 run ;

 x "&Rexepath. --no-restore --no-save < ""&prgpath."" " ;
 x """&pdfpath.""" ;

 proc import out=TE_FIXED datafile="&path./TEfixed.csv" 
  dbms=csv replace ; getnames=yes; datarow=2 ; run ;
 proc import out=TE_FIXED_LCL datafile="&path./\TEfixedLCL.csv" 
  dbms=csv replace ; getnames=yes; datarow=2 ; run ;
 proc import out=TE_FIXED_UCL datafile="&path./\TEfixedUCL.csv" 
  dbms=csv replace ; getnames=yes; datarow=2 ; run ;
 proc import out=TE_RANDOM datafile="&path./TErandom.csv" 
  dbms=csv replace ; getnames=yes; datarow=2 ; run ;
 proc import out=TE_RANDOM_LCL datafile="&path./TErandomLCL.csv" 
  dbms=csv replace ; getnames=yes; datarow=2 ; run ;
 proc import out=TE_RANDOM_UCL datafile="&path./TErandomUCL.csv" 
  dbms=csv replace ;getnames=yes; datarow=2 ; run ;
 proc import out=Q1_STATISTICS datafile="&path./Q1statistics.csv" 
  dbms=csv replace ; getnames=yes; datarow=2 ; run ;
 proc import out=Q1_WITHINDESIGNS datafile="&path./Q1withindesigns.csv" 
  dbms=csv replace ; getnames=yes; datarow=2 ; run ;
 proc import out=Q1_BETWEENDESIGNS datafile="&path./Q1betweendesigns.csv"
  dbms=csv replace ; getnames=yes; datarow=2 ; run ;
 proc import out=Q2_STATISTICS datafile="&path./Q2statistics.csv" 
  dbms=csv replace ; getnames=yes; datarow=2 ; run ;
 proc import out=Q2_WITHINDESIGNS datafile="&path./Q2withindesigns.csv" 
  dbms=csv replace ; getnames=yes; datarow=2 ; run ;
 proc import out=Q2_BETWEENDESIGNS datafile="&path./Q2betweendesigns.csv"
  dbms=csv replace ; getnames=yes; datarow=2 ; run ;
 proc import out=RANK_FIXED datafile="&path./rankfixed.csv" 
  dbms=csv replace ; getnames=yes; datarow=2 ; run ;
 proc import out=RANK_RANDOM datafile="&path./rankrandom.csv" 
  dbms=csv replace ; getnames=yes; datarow=2 ; run ;

 title "Treatment estimate (Fixed effect model)" ;
 proc print data=TE_FIXED noobs ; run ;
 title "Lower confidence limit (Fixed effect model)" ;
 proc print data=TE_FIXED_LCL noobs ; run ;
 title "Upper confidence limit (Fixed effect model)" ;
 proc print data=TE_FIXED_UCL noobs ; run ;
 title "Treatment estimate (Random effect model)" ;
 proc print data=TE_RANDOM noobs ; run ;
 title "Lower confidence limit (Random effect model)" ;
 proc print data=TE_RANDOM_LCL noobs ; run ;
 title "Upper confidence limit (Random effect model)" ;
 proc print data=TE_RANDOM_UCL noobs ; run ;
 title "Q statistics to assess homogeneity / consistency" ;
 proc print data=Q1_STATISTICS noobs ; run ;
 title "Design-specific decomposition of within-designs Q statistic" ;
 proc print data=Q1_WITHINDESIGNS noobs ; run ;
 title "Between-designs Q statistic after detaching of single designs" ;
 proc print data=Q1_BETWEENDESIGNS noobs ; run ;
 title "Q statistics to assess homogeneity / consistency (Random)";
 proc print data=Q2_STATISTICS noobs ; run ;
 title "Design-specific decomposition of within-designs Q statistic (Random)" ;
 proc print data=Q2_WITHINDESIGNS noobs ; run ;
 title "Between-designs Q statistic after detaching of single designs (Random)";
 proc print data=Q2_BETWEENDESIGNS noobs ; run ;
 title "Rank treatments in network (Fixed effect model)" ;
 proc print data=RANK_FIXED noobs ; run ;
 title "Rank treatments in network (Random effect model)" ;
 proc print data=RANK_RANDOM noobs ; run ; title "" ;
%mend ;

*************************************************;

data Senn2013_char ;
  length STUDLAB $100. ;
  input TE SETE TREAT1 $ TREAT2 $ STUDLAB $ ;
  cards ;
    -1.90  0.1414  metf  plac        DeFronzo1995
    -0.82  0.0992  metf  plac           Lewin2007
    -0.20  0.3579  metf  acar          Willms1999
    -1.34  0.1435  rosi  plac        Davidson2007
    -1.10  0.1141  rosi  plac   Wolffenbuttel1999
    -1.30  0.1268  piog  plac          Kipnes2001
    -0.77  0.1078  rosi  plac         Kerenyi2004
     0.16  0.0849  piog  metf        Hanefeld2004
     0.10  0.1831  piog  rosi          Derosa2004
    -1.30  0.1014  rosi  plac           Baksi2004
    -1.09  0.2263  rosi  plac      Rosenstock2008
    -1.50  0.1624  rosi  plac             Zhu2003
    -0.14  0.2239  rosi  metf            Yang2003
    -1.20  0.1436  rosi  sulf  Vongthavaravat2002
    -0.40  0.1549  acar  sulf           Oyama2008
    -0.80  0.1432  acar  plac           Costa1997
    -0.57  0.1291  sita  plac       Hermansen2007
    -0.70  0.1273  vild  plac          Garber2008
    -0.37  0.1184  metf  sulf            Alex1998
    -0.74  0.1839  migl  plac        Johnston1994
    -1.41  0.2235  migl  plac       Johnston1998a
     0.00  0.2339  rosi  metf             Kim2007
    -0.68  0.2828  migl  plac       Johnston1998b
    -0.40  0.4356  metf  plac  Gonzalez-Ortiz2004
    -0.23  0.3467  benf  plac          Stucci1996
    -1.01  0.1366  benf  plac          Moulin2006
    -1.20  0.3758  metf  plac          Willms1999
    -1.00  0.4669  acar  plac          Willms1999
  ;
run ;

*--- s ;
%MYNETMETA(dataset   = Senn2013_char,
           sm        = MD,
           level     = 0.95) ;

%MYNETMETA(dataset   = Senn2013_char,
           sm        = MD,
           level     = 0.95,
           reference = plac,
           seq       = %str('plac', 'acar', 'benf', 'metf', 'migl', 'piog','rosi', 'sita', 'sulf', 'vild')) ;

*************************************************;

%macro MYCONVERT(dataset = ,
                 studlab = ,
                 treat   = ,
                 event   = ,
                 n       = ,
                 mean    = ,
                 sd      = ,
                 TE      = ,
                 seTE    = ,
                 time    = ,
                 sm      = ,
                 path    = C:/temp) ;
 options noxwait xsync ;
 %let Rexepath='C:\Program Files\R\R-3.2.5\bin\R.exe' ; * R.exe ̃pXw!! ;
 %let datapath=&path./data.csv  ;
 %let prgpath =&path./program.R ;

 proc export data=&dataset. outfile="&datapath." dbms=csv replace ; run ;

 filename xxx "&prgpath." ;
 data _null_ ;
  file xxx ;
  put "library(netmeta); setwd(""&path."")" ;
  put "d <- read.csv('data.csv', head=T)" ;
  put "p1 <- pairwise(data=d, studlab=&studlab.," ;
  %if &event. ne %then %do ; put "event=list(&event.)," ; %end ;
  %if &n.     ne %then %do ; put "n=list(&n.),"         ; %end ;
  %if &mean.  ne %then %do ; put "mean=list(&mean.),"   ; %end ;
  %if &sd.    ne %then %do ; put "sd=list(&sd.),"       ; %end ;
  %if &TE.    ne %then %do ; put "TE=list(&TE.),"       ; %end ;
  %if &seTE.  ne %then %do ; put "seTE=list(&seTE.),"   ; %end ;
  %if &time.  ne %then %do ; put "time=list(&time.),"   ; %end ;
  %if &sm.    ne %then %do ; put "sm=""&sm."","         ; %end ;
  put "treat=list(&treat.))" ;
  put "p1$SETE <- p1$seTE; p1$TREAT1 <- p1$treat1" ;
  put "p1$TREAT2 <- p1$treat2; p1$STUDLAB <- p1$studlab" ;
  put "p2 <- p1[,c('TE','SETE','TREAT1','TREAT2','STUDLAB')]" ;
  put "write.csv(p2, 'result.csv', quote=F, row.names=F)" ;
 run ;

 x "&Rexepath. --no-restore --no-save < ""&prgpath."" " ;

 proc import out=&dataset._CONTRAST datafile="&path./result.csv" 
  dbms=csv replace ; getnames=yes; datarow=2 ; run ;

 title """&dataset._CONTRAST (Contrast-based)""  Converted from ""&dataset""" ;
 proc print data=&dataset._CONTRAST ; run ; title "" ; title2 "" ;
%mend ;

*************************************************;

data Parkinson ;
  length STUDY $100. ;
  input STUDY $ Group1 $ TE1 seTE1 Group2 $ TE2 seTE2 Group3 $ TE3 seTE3 ;
  cards ;
    1  1  -1.22   0.50   3  -1.53   0.44   .    .     .
    2  1  -0.70   0.28   2  -2.40   0.26   .    .     .
    3  1  -0.30   0.50   2  -2.60   0.51   4  -1.20  0.48 
    4  3  -0.24   0.27   4  -0.59   0.35   .    .     .
    5  3  -0.73   0.34   4  -0.18   0.44   .    .     .
    6  4  -2.20   0.20   5  -2.50   0.19   .    .     .
    7  4  -1.80   0.20   5  -2.10   0.25   .    .     .
  ;
run ;

%MYCONVERT(dataset = Parkinson,
           studlab = STUDY,
           treat   = %str(Group1, Group2, Group3),
           TE      = %str(TE1, TE2, TE3),
           seTE    = %str(seTE1, seTE2, seTE3)) ;

*************************************************;

