/* ******************************************************** */
/*                                                          */
/* Title:   Network Meta-Analysis using Frequentist Methods */
/*          (incomplete)                                    */
/* Version: 1.0                                             */
/* Date:    2016.05.19                                      */
/* License: GPL (>= 2)                                      */
/* Reference:                                               */
/*   Rucker G, Schwarzer G, Krahn U, Konig J (2015)         */
/*    netmeta: Network meta-analysis with R.                */
/*    R package version 0.8-0.                              */
/*                                                          */
/* ******************************************************** */

proc format ;
  value treatf   1 = "acar"
                 2 = "benf"
                 3 = "metf"
                 4 = "migl"
                 5 = "piog"
                 6 = "rosi"
                 7 = "sita"
                 8 = "sulf" 
                 9 = "vild" 
                10 = "plac" ;
  value studlabf 1 = "DeFronzo1995"
                 2 = "Lewin2007"
                 3 = "Willms1999"
                 4 = "Davidson2007"
                 5 = "Wolffenbuttel1999"
                 6 = "Kipnes2001"
                 7 = "Kerenyi2004"
                 8 = "Hanefeld2004"
                 9 = "Derosa2004"
                10 = "Baksi2004"
                11 = "Rosenstock2008"
                12 = "Zhu2003"
                13 = "Yang2003"
                14 = "Vongthavaravat2002"
                15 = "Oyama2008"
                16 = "Costa1997"
                17 = "Hermansen2007"
                18 = "Garber2008"
                19 = "Alex1998"
                20 = "Johnston1994"
                21 = "Johnston1998a"
                22 = "Kim2007"
                23 = "Johnston1998b"
                24 = "Gonzalez-Ortiz2004"
                25 = "Stucci1996"
                26 = "Moulin2006" ;
run ;

data Senn2013 ;
  input TE seTE treat1 treat2 studlab ;
  format treat1 treatf. treat2 treatf. studlab studlabf. ;
  cards ;
    -1.90   0.1414  3  10   1
    -0.82   0.0992  3  10   2
    -0.20   0.3579  3   1   3
    -1.34   0.1435  6  10   4
    -1.10   0.1141  6  10   5
    -1.30   0.1268  5  10   6
    -0.77   0.1078  6  10   7
     0.16   0.0849  5   3   8
     0.10   0.1831  5   6   9
    -1.30   0.1014  6  10  10
    -1.09   0.2263  6  10  11
    -1.50   0.1624  6  10  12
    -0.14   0.2239  6   3  13
    -1.20   0.1436  6   8  14
    -0.40   0.1549  1   8  15
    -0.80   0.1432  1  10  16
    -0.57   0.1291  7  10  17
    -0.70   0.1273  9  10  18
    -0.37   0.1184  3   8  19
    -0.74   0.1839  4  10  20
    -1.41   0.2235  4  10  21
     0.00   0.2339  6   3  22
    -0.68   0.2828  4  10  23
    -0.40   0.4356  3  10  24
    -0.23   0.3467  2  10  25
    -1.01   0.1366  2  10  26
    -1.20   0.3758  3  10   3
    -1.00   0.4669  1  10   3
  ;
run ;

*************************************************;

*--- Senn2013 ̃f[^ 7 R[ĥݒoĉ ;
data Subset ;
  set Senn2013 ;
  where studlab in (1,3,16,19,24) ;
run ;

proc iml ;
 *use work.Subset ;
 *read all var {TE studlab} into mf ;
 *close work.Subset ;
 *y = mf[{4 3 1 5 6 7},1] ;
  y = {-0.37, -0.8, -1.9, -0.4, -1.2, -1} ;

  X = {1 0 -1, 0 1 0, 1 0 0, 1 0 0, 1 0 0, 0 1 0} ;
  V = {0.01401856 0 0 0 0 0,
       0 0.02050624 0 0 0 0,
       0 0 0.01999396 0 0 0,
       0 0 0 0.19053225 0 0,
       0 0 0 0 0.14122564 0.11556442,
       0 0 0 0 0.11556442 0.21799561} ;

  theta_net   = inv(t(X) * inv(V) * X) * t(X) * inv(V) *y ;
  v_theta_net = inv(t(X) * inv(V) * X) ;
  result1     = theta_net || (theta_net-probit(0.975)#sqrt(vecdiag(v_theta_net)))
                          || (theta_net+probit(0.975)#sqrt(vecdiag(v_theta_net))) ;

  theta_dir      = j(5, 1, 0) ; Va = j(5, 5, 0) ; Xa = X[{1 2 3 5 6},] ;
  theta_dir[1,1] = sum(inv(V[1,1]) * y[1]) # V[1,1] ; Va[1,1] = V[1,1] ;
  theta_dir[2,1] = sum(inv(v[2,2]) * y[2]) # V[2,2] ; Va[2,2] = V[2,2] ;
  theta_dir[3,1] = sum(inv(V[{3 4},{3,4}]) * y[{3 4}]) / sum(vecdiag(inv(v[{3 4},{3 4}]))) ;
         Va[3,3] = 1/sum(vecdiag(inv(V[{3 4},{3 4}]))) ;
  theta_dir[{4 5},1] = y[{5 6}] ; Va[{4 5},{4 5}] = V[{5 6},{5 6}] ;

  theta_net_a   = inv(t(Xa) * inv(Va) * Xa) * t(Xa) * inv(Va) * theta_dir ;
  v_theta_net_a = inv(t(Xa) * inv(Va) * Xa) ;
  result2       = theta_net_a || (theta_net_a-probit(0.975)#sqrt(vecdiag(v_theta_net_a)))
                              || (theta_net_a+probit(0.975)#sqrt(vecdiag(v_theta_net_a))) ;

  Q_net = t(y - X * theta_net) * inv(V) * (y-X * theta_net) ;
  Q_inc = t(theta_dir - Xa * theta_net_a) * inv(Va) * (theta_dir - Xa * theta_net_a) ;
  Q_het = Q_net - Q_inc ;

  CC = -1#i(3); CC[,1] = 1; E = CC * theta_net; z = j(1,3,0) ;
  do i=1 to 3 ;
    c    = CC[i,] ;
    z[i] = sqrt(t(c * theta_net) * inv(c * v_theta_net * t(c)) * (c * theta_net)) ;
    if (E[i] < 0) then z[i] = -1*z[i] ;
  end ;
  p_metf = 1-(probnorm(z)) ; pscore_metf = p_metf[:] ;

  title "Treatment estimate (Fixed effect model)" ; 
  print result1 result2, Q_net Q_het Q_inc pscore_metf ;
quit ;



*--- Senn2013 Ŝɑ΂ Network Meta-Analysis ;
proc iml ;
  use work.Senn2013 ;
  read all var {TE seTE treat1 treat2 studlab} into mf ;
  close work.Senn2013 ;

*** ϒlʂHi1:A0:Ȃj ;
  smallIsGood = 1 ;

*** Œ ;
  TE       = mf[,1] ;
  seTE     = mf[,2] ;
  treat1   = mf[,3] ;
  treat2   = mf[,4] ;
  studlab  = mf[,5] ;
  labels   = union(treat1 // treat2) ;
  call sort(labels, 1);

  wo = choose(treat1 > treat2, 1, 0) ;
  do i=1 to nrow(wo) ;
    if (wo[i,1]=1) then do ;
      TE[i,1]      = -TE[i,1] ;
      ttreat1      =  treat1[i,1] ;
      treat1[i,1]  =  treat2[i,1] ;
      treat2[i,1]  =  ttreat1 ;
    end ;
  end ;

  weights = 1/(seTE##2) ;
  NAs     = j(nrow(treat1),1,.) ;
  d       = studlab || treat1 || treat2 || TE || seTE || weights ;
  call sort(d, {1, 2, 3}) ; * \[g ;

  sl = union(studlab) ;
  newdata = j(1, 7, .) ;
  do i=1 to ncol(sl) ;
    s        = sl[i] ;
    index    = loc(d[,1]=s) ;
    subgraph = d[index, ] ;
    subgraph = subgraph || j(nrow(subgraph), 1, (1+sqrt(8*nrow(subgraph)+1))/2) ;
    if (nrow(subgraph) > 1) then do ;
      r = 1/subgraph[,6] ;
      m = nrow(r) ;
      n = (1 + sqrt(8 * m + 1))/2 ;
      B = j(m, n, 0) ;
      k = 1 ;

      do ii=1 to (n-1) ;
        do jj=(ii+1) to n ;
          B[k,ii] =  1  ;
          B[k,jj] = -1  ;
          k       = k+1 ;
        end ;
      end ;
      R    = diag(vecdiag(t(B) * diag(r) * B)) - t(B) * diag(r) * B ;
      Lt   = -0.5 # t(B) * B * R * t(B) * B / n##2 ;
      L    = inv(Lt - 1/n) + 1/n ;
      W    = diag(vecdiag(L)) - L ;
      v    = j(1, n, 0) ;

      edge = 0 ;
      do ii=1 to (n-1) ;
        do jj=(ii+1) to n ;
          edge    = edge + 1 ;
          v[edge] = W[ii,jj] ;
        end ;
      end ;
      subgraph[,6] = t(v) ;
    end ;
    newdata = newdata // subgraph ;
  end ;

  res        = newdata ;
  p0         = res[2:nrow(res),] ;
  studlab    = p0[,1] ;
  treat1     = p0[,2] ;
  treat2     = p0[,3] ;
  TE         = p0[,4] ;
  seTE       = sqrt(1/p0[,6]) ;
  narms      = p0[,7] ;
  level      = 0.95   ;
  level_comb = 0.95   ;
  seTE_orig  = p0[,5] ;
  treat1_pos = j(nrow(treat1), 1, .) ;
  treat2_pos = j(nrow(treat2), 1, .) ;
  do i=1 to nrow(treat1) ;
    do j=1 to ncol(labels) ;
      if (treat1[i,1]=labels[1,j]) then treat1_pos[i,1]=j ;
      if (treat2[i,1]=labels[1,j]) then treat2_pos[i,1]=j ;
    end ;
  end ;

  w_pooled   = 1/seTE##2 ;
  m          = nrow(TE) ;
  n          = ncol(labels) ;
  treat_max  = max(labels) ;
  W          = diag(w_pooled) ;
  df1        = 2 # sum(1/narms) ;
  B          = j(m, treat_max, 0) ;
  do i=1 to m ;
    if (treat1[i,1] ^= treat2[i,1]) then do ;
      B[i,treat1[i,1]] =  1 ;
      B[i,treat2[i,1]] = -1 ;
    end ;
  end ;
  B = B[,labels] ;

  MM    = t(B) * B ;
  DD    = diag(vecdiag(MM)) ;
  A     = DD - MM ;
  L     = t(B) * W * B ;
  Lplus = inv(L - 1/n) + 1/n ;
  R     = j(n, n, 0) ;
  do i=1 to n ;
    do j=1 to n ;
      R[i,j] = Lplus[i,i] + Lplus[j,j] - 2#Lplus[i,j] ;
    end ;
  end ;
  VV = j(m, 1, 0) ;
  do i=1 to m ;
    VV[i] = R[treat1_pos[i], treat2_pos[i]] ;
  end ;
  G        = B * Lplus * t(B) ;
  H        = G * W ;
  leverage = vecdiag(H) ;
  v        = H * TE ;

  ci_v_TE     = v ;
  ci_v_seTE   = sqrt(VV) ;
  ci_v_lower  = v - probit(1 - (1-level)/2) * sqrt(VV) ;
  ci_v_upper  = v + probit(1 - (1-level)/2) * sqrt(VV) ;
  ci_v_zscore = v / sqrt(VV) ;
  ci_v_pval   = 2 # (1-probnorm(abs(ci_v_zscore))) ;
  ci_v_level  = level ;

  all = j(n, n, .) ;
  do i=1 to m ;
    all[treat1_pos[i,1], treat2_pos[i,1]] = v[i,1] ;
  end ;
  do i=1 to n ;
    do j=1 to n ;
      do k=1 to n ;
        if (all[i, k] ^= . & all[j, k] ^= .) then do ;
          all[i, j] = all[i, k] - all[j, k] ;
          all[j, i] = all[j, k] - all[i, k] ;
        end ;
        if (all[i, j] ^= . & all[k, j] ^= .) then do ;
          all[i, k] = all[i, j] - all[k, j] ;
          all[k, i] = all[k, j] - all[i, j] ;
        end ;
        if (all[i, k] ^= . & all[i, j] ^= .) then do ;
          all[j, k] = all[i, k] - all[i, j] ;
          all[k, j] = all[i, j] - all[i, k] ;
        end ;
      end ;
    end ;
  end ;

  QQ     = t(TE - v) * W * (TE - v) ;
  df     = df1 - (n - 1) ;
  pval_Q = 1-probchi(QQ, df) ;
  I2     = max(0, 100 * (QQ - df)/QQ) ;
  II     = i(m) ;
  E      = j(m, m, 0) ;
  do i=1 to m ;
    do j=1 to m ;
      if (studlab[i,1]=studlab[j,1]) then E[i,j]=1 ;
    end ;
  end ;

  tau2       = max(0, (QQ - df)/sum(vecdiag((II - H) * (B * t(B) # E/2) * W))) ;
  tau        = sqrt(tau2) ;
  Q_matrix   = j(n, n, 0) ;
  n_pairwise = 0 ;
  tmpdata    = treat1_pos || treat2_pos || TE || w_pooled ;
  do i=1 to (n-1) ;
    do j=(i+1) to n ;
      if (A[i,j] > 1) then do ;
        n_pairwise     = n_pairwise + 1 ;
        index          = loc((tmpdata[,1]=i & tmpdata[,2]=j) | (tmpdata[,1]=j & tmpdata[,2]=i)) ;
        sub            = tmpdata[index, ] ;
        w_fixed        = 1/(1/sqrt(sub[,4]))##2 ;
        TE_fixed       = sum(sub[,3] # w_fixed) / sum(w_fixed) ;
        Q_matrix[i, j] = sum(w_fixed # ((sub[,3] - TE_fixed)##2)) ;
      end ;
    end ;
  end ;

  q   = {.} ;
  t1  = {.} ;
  t2  = {.} ;
  dfs = {.} ;
  p   = 1 ;
  do i=1 to (n-1) ;
    do j=(i+1) to n ;
      if (A[i,j] > 0) then do ;
        p   = p + 1 ;
        t1  = t1  || i ;
        t2  = t2  || j ;
        q   = q   || (Q_matrix[i,j]) ;
        dfs = dfs || (A[i,j] - 1) ;
      end ;
    end ;
  end ;
  q   = q[1,2:p] ;
  t1  = t1[1,2:p] ;
  t2  = t2[1,2:p] ;
  dfs = dfs[1,2:p] ;
  chisq_val = j(1, ncol(q), 1) ;
  do i=1 to ncol(q); if (dfs[1,i] ^= 0) then ; chisq_val[1,i] = 1-probchi(q[1,i], dfs[1,i]) ; end ;

  Q_heterogeneity = sum(Q_matrix) ;
  Q_inconsistency = QQ - Q_heterogeneity ;
  Q_decomp        = t(labels[1,t1]) || t(labels[1,t2]) || t(q) || t(dfs) || t(chisq_val) ;
  TE_pooled       = all ;
  seTE_pooled     = sqrt(R) ;
  lower_pooled    = all - probit(1 - (1-level_comb)/2) * sqrt(R) ;
  upper_pooled    = all + probit(1 - (1-level_comb)/2) * sqrt(R) ;
  z_pooled        = all / sqrt(R) ;
  pval_pooled     = 2 # (1-probnorm(abs(z_pooled))) ;
  level_pooled    = level_comb ;

  A_matrix     = A  ;
  L_matrix     = L  ;
  Lplus_matrix = Lplus ;
  G_matrix     = G  ;
  H_matrix     = H  ;
  Bo           = B  ;

  do i=1 to nrow(Bo) ;
    do j=1 to ncol(Bo) ;
      if (Bo[i,j] = 1) then ; Bo[i,j] = -2 ;
    end ;
  end ;
  Bo = Bo || t(1:nrow(Bo)) ; 
  call sort(Bo, 1:(ncol(Bo)-1)) ;
  B_matrix     = B[t(Bo[,ncol(Bo)]), ] ;
  TE_direct    = j(ncol(labels), ncol(labels), .) ;
  seTE_direct  = j(ncol(labels), ncol(labels), .) ;

  C_matrix     = B_matrix[1,] ; * ȉAnodupkey ̏ ;
  do i=2 to nrow(B_matrix) ;
    flag = 0 ;
    tmp  = B_matrix[i,] ;
    do j=1 to ncol(tmp) ;
      if (C_matrix[nrow(C_matrix),j] ^= tmp[1,j]) then flag=1 ;
    end ;
    if (flag=1) then C_matrix = C_matrix//tmp ;
  end ;

  do i=1 to nrow(C_matrix) ;
    sel1       = loc(C_matrix[i, ]= 1) ;
    sel2       = loc(C_matrix[i, ]=-1) ;
    sel_treat1 = labels[1,sel1] ;
    sel_treat2 = labels[1,sel2] ;
    selstud    = loc(treat1[,1]=sel_treat1 & treat2[,1]=sel_treat2) ;
    TE_tmp     = TE[selstud] ;
    seTE_tmp   = seTE_orig[selstud] ;
    w_fixed    = 1/seTE_tmp##2 ;
    TE_fixed   = sum(TE_tmp # w_fixed) / sum(w_fixed) ;
    seTE_fixed = sqrt(1/sum(w_fixed)) ;

    TE_direct[sel1, sel2]   =  TE_fixed ;
    seTE_direct[sel1, sel2] =  seTE_fixed ;
    TE_direct[sel2, sel1]   = -TE_fixed ;
    seTE_direct[sel2, sel1] =  seTE_fixed ;
  end ;

  lower_direct = TE_direct - probit(1 - (1-level_comb)/2) * seTE_direct ;
  upper_direct = TE_direct + probit(1 - (1-level_comb)/2) * seTE_direct ;
  z_direct     = TE_direct / seTE_direct ;
  pval_direct  = 2 # (1-probnorm(abs(z_direct))) ;
  level_direct = level_comb ;
  Q_pooled     = w_pooled # (TE - v)##2 ;

  w = (1 + sign(TE_pooled))/2 ;
  p = pval_pooled ;
  if (smallIsGood=1) then PP = w # p/2 + (1 - w) # (1 - p/2) ;
  else                    PP = w # (1 - p/2) + (1 - w) # p/2 ;
  Rank_Fixed = PP[,:] ;

  print "Fixed Effect Model" ;
  print TE_pooled lower_pooled upper_pooled, Rank_Fixed ;

*** ϗʌ ;
  TE       = mf[,1] ;
  seTE     = mf[,2] ;
  treat1   = mf[,3] ;
  treat2   = mf[,4] ;
  studlab  = mf[,5] ;
  * tau͊ɎZo ;

  weights = 1/(seTE##2 + tau##2) ;

  NAs     = j(nrow(treat1),1,.) ;
  d       = studlab || treat1 || treat2 || TE || seTE || weights ;
  call sort(d, {1, 2, 3}) ;

  sl = union(studlab) ;
  newdata = j(1, 7, .) ;
  do i=1 to ncol(sl) ;
    s        = sl[i] ;
    index    = loc(d[,1]=s) ;
    subgraph = d[index, ] ;
    subgraph = subgraph || j(nrow(subgraph), 1, (1+sqrt(8*nrow(subgraph)+1))/2) ;
    if (nrow(subgraph) > 1) then do ;
      r = 1/subgraph[,6] ;
      m = nrow(r) ;
      n = (1 + sqrt(8 * m + 1))/2 ;
      B = j(m, n, 0) ;
      k = 1 ;

      do ii=1 to (n-1) ;
        do jj=(ii+1) to n ;
          B[k,ii] =  1  ;
          B[k,jj] = -1  ;
          k       = k+1 ;
        end ;
      end ;
      R    = diag(vecdiag(t(B) * diag(r) * B)) - t(B) * diag(r) * B ;
      Lt   = -0.5 # t(B) * B * R * t(B) * B / n##2 ;
      L    = inv(Lt - 1/n) + 1/n ;
      W    = diag(vecdiag(L)) - L ;
      v    = j(1, n, 0) ;

      edge = 0 ;
      do ii=1 to (n-1) ;
        do jj=(ii+1) to n ;
          edge    = edge + 1 ;
          v[edge] = W[ii,jj] ;
        end ;
      end ;
      subgraph[,6] = t(v) ;
    end ;
    newdata = newdata // subgraph ;
  end ;

  res        = newdata ;
  p1         = res[2:nrow(res),] ;
  studlab    = p1[,1] ;
  treat1     = p1[,2] ;
  treat2     = p1[,3] ;
  TE         = p1[,4] ;
  seTE       = sqrt(1/p1[,6]) ;
  narms      = p1[,7] ;
  level      = 0.95   ;
  level_comb = 0.95   ;
  seTE_orig  = p1[,5] ;
  treat1_pos = j(nrow(treat1), 1, .) ;
  treat2_pos = j(nrow(treat2), 1, .) ;
  do i=1 to nrow(treat1) ;
    do j=1 to ncol(labels) ;
      if (treat1[i,1]=labels[1,j]) then treat1_pos[i,1]=j ;
      if (treat2[i,1]=labels[1,j]) then treat2_pos[i,1]=j ;
    end ;
  end ;

  w_pooled   = 1/seTE##2 ;
  m          = nrow(TE) ;
  n          = ncol(labels) ;
  treat_max  = max(labels) ;
  W          = diag(w_pooled) ;
  df1        = 2 # sum(1/narms) ;
  B          = j(m, treat_max, 0) ;
  do i=1 to m ;
    if (treat1[i,1] ^= treat2[i,1]) then do ;
      B[i,treat1[i,1]] =  1 ;
      B[i,treat2[i,1]] = -1 ;
    end ;
  end ;
  B = B[,labels] ;

  MM    = t(B) * B ;
  DD    = diag(vecdiag(MM)) ;
  A     = DD - MM ;
  L     = t(B) * W * B ;
  Lplus = inv(L - 1/n) + 1/n ;
  R     = j(n, n, 0) ;
  do i=1 to n ;
    do j=1 to n ;
      R[i,j] = Lplus[i,i] + Lplus[j,j] - 2#Lplus[i,j] ;
    end ;
  end ;
  VV = j(m, 1, 0) ;
  do i=1 to m ;
    VV[i] = R[treat1_pos[i], treat2_pos[i]] ;
  end ;
  G        = B * Lplus * t(B) ;
  H        = G * W ;
  leverage = vecdiag(H) ;
  v        = H * TE ;

  ci_v_TE     = v ;
  ci_v_seTE   = sqrt(VV) ;
  ci_v_lower  = v - probit(1 - (1-level)/2) * sqrt(VV) ;
  ci_v_upper  = v + probit(1 - (1-level)/2) * sqrt(VV) ;
  ci_v_zscore = v / sqrt(VV) ;
  ci_v_pval   = 2 # (1-probnorm(abs(ci_v_zscore))) ;
  ci_v_level  = level ;

  all = j(n, n, .) ;
  do i=1 to m ;
    all[treat1_pos[i,1], treat2_pos[i,1]] = v[i,1] ;
  end ;
  do i=1 to n ;
    do j=1 to n ;
      do k=1 to n ;
        if (all[i, k] ^= . & all[j, k] ^= .) then do ;
          all[i, j] = all[i, k] - all[j, k] ;
          all[j, i] = all[j, k] - all[i, k] ;
        end ;
        if (all[i, j] ^= . & all[k, j] ^= .) then do ;
          all[i, k] = all[i, j] - all[k, j] ;
          all[k, i] = all[k, j] - all[i, j] ;
        end ;
        if (all[i, k] ^= . & all[i, j] ^= .) then do ;
          all[j, k] = all[i, k] - all[i, j] ;
          all[k, j] = all[i, j] - all[i, k] ;
        end ;
      end ;
    end ;
  end ;

  TE_pooled2    = all ;
  seTE_pooled2  = sqrt(R) ;
  lower_pooled2 = all - probit(1 - (1-level_comb)/2) * sqrt(R) ;
  upper_pooled2 = all + probit(1 - (1-level_comb)/2) * sqrt(R) ;
  z_pooled2     = all / sqrt(R) ;
  pval_pooled2  = 2 # (1-probnorm(abs(z_pooled2))) ;
  level_pooled2 = level_comb ;

  Bo = B  ;
  do i=1 to nrow(Bo) ;
    do j=1 to ncol(Bo) ;
      if (Bo[i,j] = 1) then ; Bo[i,j] = -2 ;
    end ;
  end ;
  Bo = Bo || t(1:nrow(Bo)) ; 
  call sort(Bo, 1:(ncol(Bo)-1)) ;
  B_matrix     = B[t(Bo[,ncol(Bo)]), ] ;
  TE_direct2   = j(ncol(labels), ncol(labels), .) ;
  seTE_direct2 = j(ncol(labels), ncol(labels), .) ;

  C_matrix     = B_matrix[1,] ;
  do i=2 to nrow(B_matrix) ;
    flag = 0 ;
    tmp  = B_matrix[i,] ;
    do j=1 to ncol(tmp) ;
      if (C_matrix[nrow(C_matrix),j] ^= tmp[1,j]) then flag=1 ;
    end ;
    if (flag=1) then C_matrix = C_matrix//tmp ;
  end ;

  do i=1 to nrow(C_matrix) ;
    sel1       = loc(C_matrix[i, ]= 1) ;
    sel2       = loc(C_matrix[i, ]=-1) ;
    sel_treat1 = labels[1,sel1] ;
    sel_treat2 = labels[1,sel2] ;
    selstud    = loc(treat1[,1]=sel_treat1 & treat2[,1]=sel_treat2) ;
    TE_tmp     = TE[selstud] ;
    seTE_tmp   = seTE_orig[selstud] ;
    w_fixed    = 1/seTE_tmp##2 ;
    TE_fixed   = sum(TE_tmp # w_fixed) / sum(w_fixed) ;
    seTE_fixed = sqrt(1/sum(w_fixed)) ;

    TE_direct2[sel1, sel2]   =  TE_fixed ;
    seTE_direct2[sel1, sel2] =  seTE_fixed ;
    TE_direct2[sel2, sel1]   = -TE_fixed ;
    seTE_direct2[sel2, sel1] =  seTE_fixed ;
  end ;

  lower_direct2 = TE_direct2 - probit(1 - (1-level_comb)/2) * seTE_direct2 ;
  upper_direct2 = TE_direct2 + probit(1 - (1-level_comb)/2) * seTE_direct2 ;
  z_direct2     = TE_direct2 / seTE_direct2 ;
  pval_direct2  = 2 # (1-probnorm(abs(z_direct2))) ;
  level_direct2 = level_comb ;
  Q_pooled2     = w_pooled # (TE - v)##2 ;

  w = (1 + sign(TE_pooled2))/2 ;
  p = pval_pooled2 ;
  if (smallIsGood=1) then PP = w # p/2 + (1 - w) # (1 - p/2) ;
  else                    PP = w # (1 - p/2) + (1 - w) # p/2 ;
  Rank_Random = PP[,:] ;

  print "Random Effect Model" ;
  print TE_pooled2 lower_pooled2 upper_pooled2, Rank_Random ;
quit ;
